using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Net.Mail;
using System.Text;

namespace Vevo.Shared.Domain.SystemServices
{
    public abstract class BaseNetMailSendService : BaseMailSendService
    {
        #region Private

        private SmtpClient CreateSmtpClient()
        {
            MailSettings mailSettings = GetMailSettings();

            SmtpClient smtpClient = new SmtpClient(
                mailSettings.SmtpServer,
                mailSettings.SmtpPort );

            if (mailSettings.SmtpAuthenRequired)
            {
                SetAuthentication( smtpClient );
            }
            smtpClient.EnableSsl = mailSettings.SmtpSslRequired;

            return smtpClient;
        }

        private void SetAuthentication( SmtpClient mailClient )
        {
            MailSettings mailSettings = GetMailSettings();

            System.Net.NetworkCredential smtpUserInfo = new System.Net.NetworkCredential(
                mailSettings.SmtpUserName,
                mailSettings.SmtpPassword );

            mailClient.UseDefaultCredentials = false;
            mailClient.Credentials = smtpUserInfo;
        }

        #endregion


        #region Protected

        protected abstract MailSettings GetMailSettings();

        protected override void SendMail( MailMessage mailMessage )
        {
            SmtpClient mailClient = CreateSmtpClient();
            mailClient.Send( mailMessage );
        }

        #endregion

    }
}
