using System;
using System.Collections.Generic;
using System.Text;

using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Orders;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class CartRepository : ICartRepository
    {
        #region Private

        private const string RepositoryName = "CartRepository";

        private const string GetOneSignature = "GetOne";
        private const string TableName = "Cart";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( TableName );

        private CartRepositoryNoCache _repository = new CartRepositoryNoCache();

        #endregion

        #region Constructors

        public CartRepository()
        {
        }

        public CartRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region ICartRepository Members

        public Cart CreateWhole( Cart cart )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            Cart newCart = _repository.CreateWhole( cart );
            _dataCache.ClearRepositoryCache();
            return newCart;
        }

        public Cart UpdateWhole( Cart cart )
        {
            Cart result = _repository.UpdateWhole( cart );
            _dataCache.ClearRepositoryCache();
            return result;
        }

        public void Delete( string cartID )
        {
             _repository.Delete( cartID );
            _dataCache.ClearRepositoryCache();
        }

        public Cart GetOne( string cartID )
        {
            Cart result = (Cart) _dataCache.GetValue( GetOneSignature, _cacheDependency, cartID );

            if (result == null)
            {
                result = _repository.GetOne( cartID );
                _dataCache.SetValue( GetOneSignature, result, cartID );
            }

            return result;
        }
        #endregion
    }
}
