using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Discounts;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class CouponRepository : ICouponRepository
    {
        #region Private

        private const string RepositoryName = "CouponRepository";

        private const string GetOneSignature = "GetOne";
        private const string SearchCouponSignature = "SearchCoupon";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string TableName = "Coupon";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( TableName );

        private CouponRepositoryNoCache _repository = new CouponRepositoryNoCache();

        #endregion

        #region Constructors

        public CouponRepository()
        {
        }

        public CouponRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods

        public Coupon Create( Coupon coupon )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            Coupon newCoupon = _repository.Create( coupon );
            _dataCache.ClearRepositoryCache();
            return newCoupon;
        }

        public Coupon Update( Coupon coupon )
        {
            Coupon result = _repository.Update( coupon );
            _dataCache.ClearRepositoryCache();
            return result;
        }

        public Coupon Update( Coupon coupon, string newCouponID )
        {
            Coupon result = _repository.Update( coupon, newCouponID );
            _dataCache.ClearRepositoryCache();
            return result;
        }

        public Coupon GetOne( string couponID )
        {
            Coupon result = (Coupon) _dataCache.GetValue( GetOneSignature, _cacheDependency, couponID );

            if (result == null)
            {
                result = _repository.GetOne( couponID );
                _dataCache.SetValue( GetOneSignature, result, couponID );
            }

            return result;
        }

        public void Delete( string couponID )
        {
            _repository.Delete( couponID );
            _dataCache.ClearRepositoryCache();
        }

        public void DeleteAll()
        {
            _repository.DeleteAll();
            _dataCache.ClearRepositoryCache();
        }

        public IList<Coupon> SearchCoupon(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems
            )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchCouponSignature,
                _cacheDependency,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchCoupon( sortBy, searchFilter, startIndex, endIndex, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchCouponSignature,
                    result,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<Coupon>) result.First;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue(
                   GetTableSchemaSignature, _cacheDependency
                   );

            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue(
                    GetTableSchemaSignature,
                    list
                    );
            }

            return list;
        }

        #endregion
    }
}
