﻿using System;
using System.Collections.Generic;
using System.Text;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.Stores;

namespace Vevo.Domain.Products
{
    public abstract class BaseCategory : IBaseCategory
    {
        #region Protected

        // Members for properties
        protected string _imageFile = String.Empty;
        protected int _sortOrder = 0;
        protected int _discountGroupID = 0;
        protected bool _isEnabled = true;
        protected string _other1 = String.Empty;
        protected string _other2 = String.Empty;
        protected string _other3 = String.Empty;
        protected string _other4 = String.Empty;
        protected string _other5 = String.Empty;
        protected string _rootID = "0";
        protected string _productListLayoutPath = String.Empty;
        protected bool _isAnchor = false;
        protected bool _isShowNewArrival = false;
        protected int _newArrivalAmount = 0;

        protected Culture _defaultCulture;
        protected LocaleCollection<BaseCategoryLocale> _locales;

        #endregion

        #region Public Constructors

        public BaseCategory( Culture defaultCulture )
        {
            _defaultCulture = defaultCulture;
            _locales = new LocaleCollection<BaseCategoryLocale>( this );
        }

        public BaseCategory()
        {
            _defaultCulture = new Culture();
            _locales = new LocaleCollection<BaseCategoryLocale>( this );
        }

        #endregion


        #region Public Properties

        //public static abstract IBaseCategory Null { get; }

        public virtual bool IsNull
        {
            get { return false; }
        }

        public string ImageFile { get { return _imageFile; } set { _imageFile = value; } }
        public bool IsEnabled { get { return _isEnabled; } set { _isEnabled = value; } }
        public int SortOrder { get { return _sortOrder; } set { _sortOrder = value; } }
        public int DiscountGroupID { get { return _discountGroupID; } set { _discountGroupID = value; } }
        public string Other1 { get { return _other1; } set { _other1 = value; } }
        public string Other2 { get { return _other2; } set { _other2 = value; } }
        public string Other3 { get { return _other3; } set { _other3 = value; } }
        public string Other4 { get { return _other4; } set { _other4 = value; } }
        public string Other5 { get { return _other5; } set { _other5 = value; } }
        public string RootID { get { return _rootID; } set { _rootID = value; } }
        public string ProductListLayoutPath { get { return _productListLayoutPath; } set { _productListLayoutPath = value; } }
        public bool IsAnchor { get { return _isAnchor; } set { _isAnchor = value; } }
        public bool IsShowNewArrival { get { return _isShowNewArrival; } set { _isShowNewArrival = value; } }
        public int NewArrivalAmount { get { return _newArrivalAmount; } set { _newArrivalAmount = value; } }
        public Culture DefaultCulture { get { return _defaultCulture; } set { _defaultCulture = value; } }
        public LocaleCollection<BaseCategoryLocale> Locales { get { return _locales; } }

        //------------------------------------------------------------
        // Locale items
        public string Name
        {
            get { return Locales[DefaultCulture].Name; }
            set { Locales.GetNoLoad( DefaultCulture ).Name = value; }
        }
        public string Description
        {
            get { return Locales[DefaultCulture].Description; }
            set { Locales.GetNoLoad( DefaultCulture ).Description = value; }
        }
        public string MetaKeyword
        {
            get { return Locales[DefaultCulture].MetaKeyword; }
            set { Locales.GetNoLoad( DefaultCulture ).MetaKeyword = value; }
        }
        public string MetaDescription
        {
            get { return Locales[DefaultCulture].MetaDescription; }
            set { Locales.GetNoLoad( DefaultCulture ).MetaDescription = value; }
        }
        public string ImageAlt
        {
            get { return Locales[DefaultCulture].ImageAlt; }
            set { Locales.GetNoLoad( DefaultCulture ).ImageAlt = value; }
        }
        public string ImageTitle
        {
            get { return Locales[DefaultCulture].ImageTitle; }
            set { Locales.GetNoLoad( DefaultCulture ).ImageTitle = value; }
        }
        public string UrlName
        {
            get { return Locales[ DefaultCulture ].UrlName; }
            set { Locales.GetNoLoad( DefaultCulture ).UrlName = value; }
        }
        // End Locale items
        //------------------------------------------------------------

        #endregion



        #region IClonable Members

        public abstract object Clone();

        #endregion

        #region ILocaleLoader Members

        public abstract IList<ILocale> GetLocales();

        #endregion


        #region Public Methods

        public abstract IList<IBaseCategory> GetBreadcrumb();

        public string GetPageTitle( Culture culture, string defaultValue )
        {
            if (String.IsNullOrEmpty( defaultValue ))
                return _locales[culture].Name;
            else
                return defaultValue;
        }

        public string GetMetaDescription( Culture culture, string defaultValue )
        {
            if (!String.IsNullOrEmpty( Locales[culture].MetaDescription ))
                return Locales[culture].MetaDescription;
            else
                return defaultValue;
        }

        public string GetMetaKeyword( Culture culture, string defaultValue )
        {
            if (!String.IsNullOrEmpty( Locales[culture].MetaKeyword ))
                return Locales[culture].MetaKeyword;
            else
                return defaultValue;
        }

        public bool IsCategoryAvailableStore( string storeID )
        {
            string rootID = DataAccessContext.Configurations.GetValue( "RootCategory", DataAccessContext.StoreRepository.GetOne( storeID ) );
            if (rootID == RootID)
                return true;
            else
                return false;
        }

        public abstract bool IsParentsEnable();

        public IList<string> GetAvailableStoreIDList()
        {
            IList<Store> storeList = DataAccessContext.StoreRepository.GetAll( "StoreID" );
            IList<string> storeIDList = new List<string>();

            for (int i = 0; i < storeList.Count; i++)
            {
                if (storeList[i].StoreID != "0")
                {
                    if (RootID ==
                        DataAccessContext.Configurations.GetValue( "RootCategory", storeList[i] ))
                    {
                        storeIDList.Add( storeList[i].StoreID );
                    }
                    if (RootID ==
                        DataAccessContext.Configurations.GetValue( "RootDepartment", storeList[i] ))
                    {
                        storeIDList.Add( storeList[i].StoreID );
                    }
                }
            }

            return storeIDList;
        }

        #endregion
    }
}
