using System;
using System.Data;
using System.Configuration;

using Vevo.Domain.Configurations;
using Vevo.Domain.Orders;
using Vevo.Domain.Users;

namespace Vevo.Domain.Shipping
{
    /// <summary>
    /// Summary description for NullShippingMethod
    /// </summary>
    public class NullShippingMethod : ShippingMethod
    {
        #region Private

        private static NullShippingMethod _instance;


        private NullShippingMethod()
            : base( ShippingOption.Null )
        {
        }

        #endregion


        #region Public Properties

        public static NullShippingMethod Instance
        {
            get
            {
                if (_instance == null)
                    _instance = new NullShippingMethod();

                return _instance;
            }
        }

        public override bool IsNull
        {
            get { return true; }
        }

        public override string ShippingID
        {
            get
            {
                return "0";
            }
        }

        public override string Name
        {
            get
            {
                return String.Empty;
            }
            set
            {
            }
        }

        #endregion


        #region Public Methods

        public override decimal GetShippingCost(
            CartItemGroup cartItemGroup,
            WholesaleStatus wholesaleStatus,
            decimal discount )
        {
            return 0;
        }

        public override decimal GetHandlingFee( CartItemGroup cartItemGroup, WholesaleStatus wholesaleStatus )
        {
            return 0;
        }

        #endregion

    }
}
