using System;
using System.Data;
using System.Configuration;

using Vevo.Domain.Configurations;
using Vevo.Domain.Shipping.RealTime;

namespace Vevo.Domain.Shipping.Custom
{
    /// <summary>
    /// Summary description for ShippingFactory
    /// </summary>
    public class ShippingFactory
    {
        public static ShippingMethod CreateShipping(
            string shippingID )
        {
            ShippingOption shippingOption = DataAccessContext.ShippingOptionRepository.GetOne(
                Culture.Null, shippingID );

            if (shippingOption.ShippingOptionType.IsRealTime)
            {
                return new RealTimeShippingMethod( shippingOption, null );
            }
            else
            {
                switch (shippingOption.ShippingOptionType.TypeName)
                {
                    case "Fixed":
                        return new FixedShippingMethod( shippingOption, null );

                    case "PerItem":
                        return new PerItemShippingMethod( shippingOption, null );

                    case "ByWeight":
                        return new ByWeightShippingMethod( shippingOption, null );

                    default:
                        return ShippingMethod.Null;
                }
            }
        }
    }
}
