using System;
using System.Data;
using System.Configuration;
using Vevo.Data;
using Vevo.DataAccessLib;
using Vevo.WebUI.DataAccessCache;

namespace Vevo.WebUI.International
{
    /// <summary>
    /// Summary description for ReplacementRule
    /// </summary>
    public class LanguageTokenRule
    {
        private string _pagePath;
        private string _pageID;
        private string _currentCultureID;
        private string _baseCultureID;

        private DataTable _languageTable;
        private DataTable _baseLanguageTable;


        private string FindTextInPage( DataTable table, string pageID, string cultureID, string token )
        {
            DataRow row = table.Rows.Find(
                new object[] { pageID, cultureID, token } );

            if (row != null)
                return row["TextData"].ToString();
            else
                return null;
        }

        private string FindTextCurrentCulture( string token )
        {
            string text = FindTextInPage( _languageTable, _pageID, _currentCultureID, token );

            // Search for global list if local list does not contain token
            if (text == null)
                text = FindTextInPage( _languageTable, LanguageTextAccess.GlobalID, _currentCultureID, token );

            return text;
        }

        private string FindTextBaseCulture( string token )
        {
            string text = FindTextInPage( _baseLanguageTable, _pageID, _baseCultureID, token );

            // Search for global list if local list does not contain token
            if (text == null)
                text = FindTextInPage( _baseLanguageTable, LanguageTextAccess.GlobalID, _baseCultureID, token );

            return text;
        }


        public LanguageTokenRule( string pagePath, string currentCultureID, string baseCultureID )
        {
            _pagePath = pagePath;
            _currentCultureID = currentCultureID;
            _baseCultureID = baseCultureID;

            _pageID = PageAccessCache.Instance.GetIDByPath( _pagePath );

            if (_pageID != null)
            {
                _languageTable = LanguageTextAccessCache.Instance.GetTextByPage( _pageID, currentCultureID );
                _baseLanguageTable = LanguageTextAccessCache.Instance.GetTextByPage( _pageID, baseCultureID );
            }
        }

        public string GetReplacementText( string token )
        {
            if (_pageID != null)
            {
                string text = FindTextCurrentCulture( token );

                if (String.IsNullOrEmpty( text ) &&
                    _currentCultureID != _baseCultureID)
                    text = FindTextBaseCulture( token );

                return text;
            }
            else
            {
                return null;
            }
        }

    }
}