﻿using System;
using Vevo.Deluxe.Domain.BundlePromotion;
using Vevo.Domain;
using Vevo.Domain.Orders;
using Vevo.Domain.Products;
using Vevo.Domain.Stores;
using Vevo.Shared.Utilities;
using Vevo.Deluxe.Domain.Products;

namespace Vevo.WebUI.Orders
{
    public class CartAddItemService
    {
        #region Private

        private Culture _culture;
        private Cart _cart;

        private bool IsOutOfStock( string productID, int purchaseQuantity, out int currentStock )
        {
            Product product = DataAccessContext.ProductRepository.GetOne(
                _culture, productID, new StoreRetriever().GetCurrentStoreID() );
            currentStock = product.GetPurchasableStock(
                 _cart.GetNumberOfItems( productID )
                 + PromotionSelectedItem.FindSubProductAmountInPromotion(
                    _cart, productID ) );
            return product.IsOutOfStock(
                purchaseQuantity,
                _cart.GetNumberOfItems( productID )
                + PromotionSelectedItem.FindSubProductAmountInPromotion(
                    _cart, productID ) );
        }

        private bool IsOutOfStock( 
            string productID, 
            int purchaseQuantity, 
            out int currentStock, 
            out string errorCurrentStock )
        {
            Product product = DataAccessContext.ProductRepository.GetOne( 
                _culture, productID, new StoreRetriever().GetCurrentStoreID() );
            currentStock = product.GetPurchasableStock(
                 _cart.GetNumberOfItems( productID ) 
                 + PromotionSelectedItem.FindSubProductAmountInPromotion( _cart, productID ) );
            if (product.IsOutOfStock(
                purchaseQuantity,
                _cart.GetNumberOfItems( productID ) 
                + PromotionSelectedItem.FindSubProductAmountInPromotion( _cart, productID ) ))
            {
                errorCurrentStock = product.Name;
                return true;
            }
            else
            {
                errorCurrentStock = String.Empty;
                return false;
            }
        }

        private bool VerifyStockOption(
            string productID, 
            string[] optionItemIDs, 
            int quantity, 
            out int currentStock, 
            string storeID )
        {
            return VerifyStockOption( 
                productID, optionItemIDs, quantity, String.Empty, out currentStock, storeID );
        }

        private bool VerifyStockOption(
            string productID, 
            string[] optionItemIDs, 
            int qty, 
            string name, 
            out int currentStock, 
            string storeID )
        {
            Product product = DataAccessContext.ProductRepository.GetOne( _culture, productID, storeID );

            if (optionItemIDs == null || optionItemIDs.Length == 0)
            {
                if (IsOutOfStock( productID, qty, out currentStock ))
                {
                    return false;
                }
                else
                {
                    return true;
                }
            }
            else
            {
                string combinationID = product.FindProductStock( optionItemIDs ).OptionCombinationID;
                currentStock = product.GetPurchasableStock(
                    _cart.GetNumberOfItems( productID, combinationID )
                    + PromotionSelectedItem.FindSubProductAmountInPromotion(
                        _cart, productID, combinationID ),
                    optionItemIDs );
                return !product.IsOutOfStock(
                    qty,
                    _cart.GetNumberOfItems( productID, combinationID )
                    + PromotionSelectedItem.FindSubProductAmountInPromotion(
                        _cart, productID, combinationID ),
                    optionItemIDs
                    );
            }
        }

        private bool VerifyStockOptionWithInputList(
            string productID,
            OptionItemValueCollection inputList,
            OptionItemValueCollection optionsWithoutInputList,
            out string errorCurrentStock,
            out int currentStock
            )
        {
            bool verify = true;
            errorCurrentStock = String.Empty;
            currentStock = 0;
            foreach (OptionItemValue input in inputList)
            {
                OptionItemValueCollection optionsWithStock = 
                    optionsWithoutInputList.GetUseStockOptionItems();

                if (input.UseStock)
                {
                    optionsWithStock.Add( input );
                }

                if (optionsWithStock.Count > 0)
                {
                    if (!VerifyStockOption(
                        productID,
                        optionsWithStock.ConvertToIDArray(),
                        ConvertUtilities.ToInt32( input.Details ),
                        input.Name,
                        out currentStock, new StoreRetriever().GetCurrentStoreID() ))
                    {
                        errorCurrentStock = input.Name;
                        verify = false;
                        break;
                    }
                }
            }

            return verify;
        }

        #endregion


        #region Public Constructors

        public CartAddItemService( Culture culture, Cart cart )
        {
            _culture = culture;
            _cart = cart;
        }

        #endregion


        #region Public Methods

        public bool AddToCart(
            Product product,
            OptionItemValueCollection selectedOptions,
            ProductKitItemValueCollection valueCollection,
            int quantity,
            CartItemGiftDetails giftDetails,
            decimal productCustomPrice,
            out string errorCurrentStock,
            out int currentStock )
        {
            errorCurrentStock = String.Empty;
            bool result = true;

            if (product.IsProductKit)
            {
                foreach (ProductKitItemValue item in valueCollection)
                {
                    if (IsOutOfStock( 
                        item.ProductID, 
                        quantity * item.Quantity, 
                        out currentStock, 
                        out errorCurrentStock ))
                    {
                        return false;
                    }
                }
            }

            if (selectedOptions == null || selectedOptions.Count == 0)
            {
                if (IsOutOfStock( product.ProductID, quantity, out currentStock ))
                {
                    result = false;
                }
                else
                {
                    if (product.IsProductKit)
                    {
                        if (product.IsCustomPrice)
                        {
                            _cart.AddProductKitItem( 
                                product, quantity, OptionItemValueCollection.Null, valueCollection, productCustomPrice );
                        }
                        else
                        {
                            _cart.AddProductKitItem( 
                                product, quantity, OptionItemValueCollection.Null, valueCollection );
                        }
                    }
                    else
                    {
                        ProductSubscription productSubscription = 
                            new ProductSubscription( product.ProductID );

                        if (product.IsGiftCertificate)
                        {
                            _cart.AddItem(
                                product, 
                                quantity, 
                                giftDetails, 
                                productSubscription.IsSubscriptionProduct() );
                        }
                        else if (product.IsCustomPrice)
                        {
                            _cart.AddItem( 
                                product, 
                                quantity, 
                                productCustomPrice, 
                                productSubscription.IsSubscriptionProduct() );
                        }
                        else
                        {
                            _cart.AddItem( 
                                product, 
                                quantity, 
                                productSubscription.IsSubscriptionProduct() );
                        }
                    }
                }
            }
            else
            {
                OptionItemValueCollection inputLists = 
                    selectedOptions.GetOptionItemsByGroupType( OptionGroup.OptionGroupType.InputList );
                if (inputLists.Count == 0)
                {
                    OptionItemValueCollection optionsWithStock = selectedOptions.GetUseStockOptionItems();
                    if (VerifyStockOption(
                        product.ProductID, 
                        optionsWithStock.ConvertToIDArray(), 
                        quantity, 
                        out currentStock, 
                        new StoreRetriever().GetCurrentStoreID() ))
                    {
                        if (product.IsProductKit)
                        {
                            _cart.AddProductKitItem( 
                                product, quantity, selectedOptions, valueCollection, productCustomPrice );
                        }
                        else
                        {
                            _cart.AddItem(
                                product, quantity, selectedOptions, productCustomPrice );
                        }
                    }
                    else
                        result = false;
                }
                else
                {
                    OptionItemValueCollection optionsWithoutInputList =
                        selectedOptions.GetOptionItemsNotInGroupType( 
                            OptionGroup.OptionGroupType.InputList );

                    result = VerifyStockOptionWithInputList( 
                        product.ProductID, 
                        inputLists,
                        optionsWithoutInputList, 
                        out errorCurrentStock, 
                        out currentStock );

                    if (result)
                    {
                        // add product is Loop.
                        foreach (OptionItemValue optionInput in inputLists)
                        {
                            OptionItemValueCollection options = optionsWithoutInputList.Clone();
                            options.Add( optionInput );

                            if (product.IsProductKit)
                            {
                                _cart.AddProductKitItem( 
                                    product, 
                                    int.Parse( optionInput.Details ), 
                                    options, 
                                    valueCollection, 
                                    productCustomPrice );
                            }
                            else
                            {
                                _cart.AddItem(
                                    product, 
                                    int.Parse( optionInput.Details ), 
                                    options, 
                                    productCustomPrice );
                            }
                        }
                    }
                }
            }

            return result;
        }

        public bool AddToCartByAdmin(
            Product product,
            OptionItemValueCollection selectedOptions,
            int quantity,
            CartItemGiftDetails giftDetails,
            decimal productCustomPrice,
            decimal enterPrice,
            string storeID,
            out string errorCurrentStock,
            out int currentStock,
            bool isSubscriptionable )
        {
            errorCurrentStock = String.Empty;
            bool result = true;
            if (selectedOptions == null || selectedOptions.Count == 0)
            {
                if (IsOutOfStock( product.ProductID, quantity, out currentStock ))
                {
                    result = false;
                }
                else
                {
                    if (product.IsGiftCertificate)
                    {
                        _cart.AddItemByAdmin( 
                            product, 
                            quantity, 
                            OptionItemValueCollection.Null, 
                            giftDetails, 
                            0, 
                            0, 
                            isSubscriptionable );
                    }
                    else if (product.IsCustomPrice)
                    {
                        _cart.AddItemByAdmin(
                            product, 
                            quantity, 
                            OptionItemValueCollection.Null, 
                            CartItemGiftDetails.Null, 
                            productCustomPrice, 
                            0, 
                            isSubscriptionable );
                    }
                    else if (enterPrice != 0.0m)
                    {
                        _cart.AddItemByAdmin(
                            product, 
                            quantity, 
                            OptionItemValueCollection.Null, 
                            CartItemGiftDetails.Null, 
                            0, 
                            enterPrice, 
                            isSubscriptionable );
                    }
                    else
                        _cart.AddItem( product, quantity, isSubscriptionable );
                }
            }
            else
            {
                OptionItemValueCollection inputLists =
                    selectedOptions.GetOptionItemsByGroupType( OptionGroup.OptionGroupType.InputList );
                if (inputLists.Count == 0)
                {
                    OptionItemValueCollection optionsWithStock = selectedOptions.GetUseStockOptionItems();
                    if (VerifyStockOption(
                        product.ProductID, optionsWithStock.ConvertToIDArray(), quantity, out currentStock, storeID ))
                    {
                        if (product.IsGiftCertificate)
                        {
                            _cart.AddItem( product, quantity, selectedOptions, giftDetails );
                        }
                        else
                        {
                            _cart.AddItemByAdmin(
                                product,
                                quantity,
                                selectedOptions,
                                CartItemGiftDetails.Null,
                                productCustomPrice,
                                enterPrice,
                                false );
                        }
                    }
                    else
                        result = false;
                }
                else
                {
                    OptionItemValueCollection optionsWithoutInputList =
                        selectedOptions.GetOptionItemsNotInGroupType( 
                            OptionGroup.OptionGroupType.InputList );

                    result = VerifyStockOptionWithInputList( product.ProductID, inputLists,
                        optionsWithoutInputList, out errorCurrentStock, out currentStock );

                    if (result)
                    {
                        // add product is Loop.
                        foreach (OptionItemValue optionInput in inputLists)
                        {
                            OptionItemValueCollection options = optionsWithoutInputList.Clone();
                            options.Add( optionInput );

                            _cart.AddItemByAdmin(
                                product,
                                int.Parse( optionInput.Details ),
                                options,
                                CartItemGiftDetails.Null,
                                productCustomPrice,
                                enterPrice,
                                false );
                        }
                    }
                }
            }

            return result;
        }

        #endregion
    }
}
