using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using Vevo.Domain.Configurations;
using Vevo.WebAppLib;
using Vevo.Domain;
using Vevo.Domain.Stores;

namespace Vevo.WebUI.Widget
{
    public class WidgetDirector
    {
        #region private
        private WidgetBuilder[] widgetBuilderSupport;
        private enum ValueType { Single, MultiCulture }

        private Configuration CreateConfigConditionally(
            ValueType valueType,
            string cultureID,
            string name,
            string itemValue,
            string mainGroup,
            string subGroup,
            string description,
            string keyword,
            string displayName,
            bool searchable,
            Configuration.ConfigurationType configType,
            string selectionValues,
            string selectionNames,
            string selectionImages,
            string parentConfigName,
            string displayUserControl,
            string enablerConfigName,
            string fieldValidation,
            string storeID
            )
        {
            Configuration config = new Configuration( name );
            ConfigurationValue value;
            if (valueType == ValueType.Single)
                value = new ConfigurationValue( "0", "0" );
            else
                value = new ConfigurationValue( cultureID, storeID );
            value.ItemValue = itemValue;
            config.Values.Add( value );

            ConfigurationDescription configDescription
                = new ConfigurationDescription( cultureID );
            configDescription.Description = description;
            configDescription.Keyword = keyword;
            configDescription.DisplayName = displayName;
            configDescription.MainGroup = mainGroup;
            configDescription.SubGroup = subGroup;
            config.Descriptions.Add( configDescription );

            config.Searchable = searchable;
            config.ConfigType = configType;
            config.SelectionValues = selectionValues;
            config.SelectionNames = selectionNames;
            config.SelectionImages = selectionImages;
            config.ParentConfigName = parentConfigName;
            config.DisplayUserControl = displayUserControl;
            config.EnablerConfigName = enablerConfigName;
            config.FieldValidation = fieldValidation;
            return config;
        }

        private Configuration CreateConfigConditionally(
            ValueType valueType,
            string cultureID,
            string name,
            string itemValue,
            string mainGroup,
            string subGroup,
            string description,
            string storeID )
        {
            Configuration config = new Configuration( name );
            ConfigurationValue value;
            if (valueType == ValueType.Single)
                value = new ConfigurationValue( "0", "0" );
            else
                value = new ConfigurationValue( cultureID, storeID );

            value.ItemValue = itemValue;
            config.Values.Add( value );

            ConfigurationDescription configDescription
                = new ConfigurationDescription( cultureID );
            configDescription.Description = description;
            configDescription.MainGroup = mainGroup;
            configDescription.SubGroup = subGroup;
            config.Descriptions.Add( configDescription );

            return config;
        }
        #endregion

        #region public
        public WidgetDirector()
        {
            widgetBuilderSupport = new WidgetBuilder[] { 
                new AddThisBuilder(), 
                new LivePersonBuilder(), 
                new LikeBoxBuilder() };
        }
        public void Construct( WidgetBuilder WidgetBuilder )
        {
            WidgetBuilder.BuildCode();
        }

        public WidgetBuilder[] WidgetBuilderSupport
        {
            get
            {
                return widgetBuilderSupport;
            }
        }

        public string[] WidgetConfigurationNameAll
        {
            get
            {
                ArrayList configList = new ArrayList();
                for (int i = 0; i < widgetBuilderSupport.Length; i++)
                {
                    configList.Add( widgetBuilderSupport[i].Widget.ConfigNameIsEnabled );
                    configList.Add( widgetBuilderSupport[i].Widget.ConfigNameCode );
                    configList.Add( widgetBuilderSupport[i].Widget.ConfigNameParameterValue );
                    configList.Add( widgetBuilderSupport[i].Widget.ConfigNameType );
                }
                return (string[]) configList.ToArray( typeof( string ) );
            }
        }

        public ConfigurationCollection WidgetConfigurationCollection
        {
            get
            {
                ConfigurationCollection configCollection = new ConfigurationCollection();
                for (int i = 0; i < widgetBuilderSupport.Length; i++)
                {
                    WidgetBuilder widgetBuilder = widgetBuilderSupport[i];

                    Configuration configEnable = CreateConfigConditionally(
                        ValueType.Single, CultureIDs.English,
                        widgetBuilder.Widget.ConfigNameIsEnabled, "False", "System", "Display",
                        "This option enables or disables " + widgetBuilder.Widget.Name, "", "Enable " + widgetBuilder.Widget.Name, true,
                        Configuration.ConfigurationType.UserControl, "", "", "", "",
                        "Components/SiteConfig/WidgetDetails.ascx", "", "", Store.Null.StoreID );

                    Configuration configType = CreateConfigConditionally(
                        ValueType.Single, CultureIDs.English,
                        widgetBuilder.Widget.ConfigNameType, "Default", "System", "System",
                        "This option is set for code type selection.", "", "Select code type", false,
                        Configuration.ConfigurationType.DropDown, "Default|Custom", "Default|Custom", "", "", "", "", "", Store.Null.StoreID );

                    Configuration configCode = CreateConfigConditionally(
                        ValueType.Single, CultureIDs.English,
                        widgetBuilder.Widget.ConfigNameCode, "", "System", "Display",
                        "This is custom code for widget.", Store.Null.StoreID );

                    Configuration configParameter = CreateConfigConditionally(
                        ValueType.Single, CultureIDs.English,
                        widgetBuilder.Widget.ConfigNameParameterValue, "", "System", "Display",
                        "This are parameter values.", Store.Null.StoreID );

                    configCollection.Add( widgetBuilder.Widget.ConfigNameIsEnabled, configEnable );
                    configCollection.Add( widgetBuilder.Widget.ConfigNameType, configType );
                    configCollection.Add( widgetBuilder.Widget.ConfigNameCode, configCode );
                    configCollection.Add( widgetBuilder.Widget.ConfigNameParameterValue, configParameter );
                }
                return configCollection;
            }
        }
        #endregion
    }
}
